package cn.wustlinghang.wusthelper.web.service.cookie;

import cn.wustlinghang.wusthelper.web.entity.CookieType;
import cn.wustlinghang.wusthelper.web.service.campus.undergrad.UndergradCookieService;
import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import java.util.concurrent.TimeUnit;

@Service
@Scope("singleton")
public class CookieManager {
    private static final Cache<String, String> cookiePool = Caffeine.newBuilder()
            .expireAfterWrite(2, TimeUnit.HOURS)
            .initialCapacity(256)
            .maximumSize(2048)
            .build();

    private final UndergradCookieService undergradCookieService;

    public CookieManager(UndergradCookieService undergradCookieService) {
        this.undergradCookieService = undergradCookieService;
    }

    public String getCookie(String username, CookieType cookieType) {
        return getCookie(username, cookieType, false);
    }

    public String getCookie(String username, CookieType cookieType, boolean forceRefresh) {
        String cacheKey = String.format("%s:%d", username, cookieType.getCode());
        String cookie = cookiePool.getIfPresent(cacheKey);

        boolean valid = cookie != null && checkCookie(cookie, cookieType) && !forceRefresh;
        if (valid) {
            return cookie;
        }

        cookie = this.refreshCookie(username, cookieType);
        cookiePool.put(cacheKey, cookie);

        return cookie;
    }

    public boolean checkCookie(String cookie, CookieType cookieType) {
        return switch (cookieType) {
            case UNDERGRAD -> undergradCookieService.checkCookie(cookie);
            case GRADUATE -> undergradCookieService.checkCookie(cookie);
            default -> false;
        };
    }

    public String refreshCookie(String username, CookieType cookieType) {
        String password = "";
        return switch (cookieType) {
            case UNDERGRAD -> undergradCookieService.getLoginCookie(username, password);
            case GRADUATE -> undergradCookieService.getLoginCookie(username, password);
            default -> null;
        };
    }
}